# -*- coding: utf-8 -*-

from __future__ import unicode_literals

"""Project metadata

Information describing the project.
"""

# The package name, which is also the "UNIX name" for the project.
package = 'pybitbucket_fork'
project = "Fork of PyBitbucket"
project_no_spaces = project.replace(' ', '')
version = '0.12.1'
description = 'Fork of a Python wrapper for the Bitbucket API'
authors = ['Ian Buchanan', 'Bernard Pratz']
authors_string = ', '.join(authors)
emails = ['ibuchanan@atlassian.com', 'z+pybitbucket+pub@m0g.net']
license = 'Apache'
copyright = '2015 ' + 'Atlassian'
url = 'https://github.com/guyzmo/pybitbucket'
bugtrack_url = 'https://github.com/guyzmo/pybitbucket/issues'
