# -*- coding: utf-8 -*-

from __future__ import unicode_literals

"""A Python wrapper for the Bitbucket API"""

from pybitbucket import metadata

__version__ = metadata.version
__author__ = metadata.authors[0]
__license__ = metadata.license
__copyright__ = metadata.copyright
